#!/bin/sh

#==============================================================================
# FILE        : logger.sh
#
# USAGE       : NA
#
# DESCRIPTION : This script contains the MiniOS Log interfaces and generic
#               Utility functions.
#
# OPTIONS     : None
# REQUIREMENT : Generic requirement.
# NOTES       : None
# COMPANY     : XFusion Technologies.
# CREATED     : <01-22-2022>
# REVISION    : <01-22-2022> - Created this file.
# BUGS=========================================================================
# Date       Modify Detail           Developer               Bug details
#==============================================================================

# normal
export NORMAL=0
# abnormal
export ABNORMAL=1
# parameter error or process exit
export PARA_OR_PROC_ERR=2
# no space
export NO_SPACE_ERR=3
# can't create dir
export DIR_CREATE_FAILED=4

export OS_LOG_PATH
#const value of different loglevel
CON_ERROR="0"
CON_WARN="1"
CON_INFO="2"
CON_DEBUG="3"

Default_Level="2"
LOG_LEVEL=${Default_Level}


#==============================================================================
# Function Name: g_LOG_Init
# Description  : MiniOS Init Log interface
# Parameter    : 1) move logfile? default is move, 0 = no move
# Date         : <03-03-2014>
#==============================================================================
function g_LOG_Init()
{
    local logsavefile=$3
    if [  -n "$logsavefile" ];then
        OS_LOG_PATH="$logsavefile"
    else
        OS_LOG_PATH="/var/log/osHealthCheck"
    fi
    UHLOG_FILE=${OS_LOG_PATH}/osHealthCheck.log
    RESULT_DIR=${OS_LOG_PATH}/result
    REPORT_FILE=${OS_LOG_PATH}/report.xml
    local loglevel=$1

    mountdiskused=`df /var/log | grep -v Filesystem | awk '{print int($5)}'`
    if [ $mountdiskused -eq 100 ]; then
       echo "[Warning]:No space left on /var/log !"
       exit ${NO_SPACE_ERR}
    fi

    if [ ! -d ${OS_LOG_PATH} ]
    then
        mkdir -p ${OS_LOG_PATH}
        chmod 700 ${OS_LOG_PATH}
        if [ $? -ne 0 ]; then
            logger -it osHealthCheck -p error "Dir ${OS_LOG_PATH} created failed!"
            exit ${DIR_CREATE_FAILED}
        fi
    fi

    #set loglevel
    if [ -z "${loglevel}" ]
    then
        LOG_LEVEL=${Default_Level}
    elif [ ! `echo ${loglevel} | grep '^[0-3]$'` ]
    then
        logger -it osHealthCheck -p warning "Warning: input invalid ! the loglevel will be set to default!"
        LOG_LEVEL=${Default_Level}
    else
        LOG_LEVEL=${loglevel}
    fi

    return 0
}

#==============================================================================
# Function Name: LOG_WriteLog
# Description  : MiniOS Log interface for installation
# Parameter    : 1) Severity of the log
#                2) Line number of log message
#                3) Log message
# Date         : <03-03-2014>
#==============================================================================
function LOG_WriteLog()
{
    local ret_code=0
    local progname=`basename ${BASH_SOURCE[2]}`
    local sysdate=`date "+%b %d %Y %H:%M:%S"`
    local severity=$1
    local line_no=$2
    shift 2
    local logmsg=$@

    if [ ${severity} -le ${LOG_LEVEL} ]
    then
        case ${severity} in
            0)  # Error log
                logger -it osHealthCheck -p error ${progname} "[line_no=${line_no}: ${logmsg}]"
                ;;
            1)  # Warning log
                logger -it osHealthCheck -p warning ${progname} "[line_no=${line_no}: ${logmsg}]"
                ;;
            2)  # Information log
               logger -it osHealthCheck -p info ${progname} "[line_no=${line_no}: ${logmsg}]"
                ;;
            3)  # Debug log
                logger -it osHealthCheck -p debug ${progname} "[line_no=${line_no}: ${logmsg}]"
                ;;
            *)  # Any other logs
                logger -it osHealthCheck -p warning "loglevel unknown"
                ;;
        esac
    ret_code=$?
    fi
    return $ret_code
}

#==============================================================================
# Function Name: g_LOG_Error
# Description  : MiniOS Error Log interface
# Parameter    : 1) Error message
# Date         : <03-03-2014>
#==============================================================================
function g_LOG_Error()
{
    LOG_WriteLog ${CON_ERROR} ${LINENO} "$@"
    return $?
}

#==============================================================================
# Function Name: g_LOG_Warn
# Description  : MiniOS Warning Log interface
# Parameter    : 1) Warning message
# Date         : <03-03-2014>
#==============================================================================
function g_LOG_Warn()
{
    LOG_WriteLog ${CON_WARN} ${LINENO} "$@"
    return $?
}

#==============================================================================
# Function Name: g_LOG_Info
# Description  : MiniOS Information Log interface
# Parameter    : 1) Info  message
# Date         : <03-03-2014>
#==============================================================================
function g_LOG_Info()
{
    LOG_WriteLog ${CON_INFO} ${LINENO} "$@"
    return $?
}

#==============================================================================
# Function Name: g_LOG_Debug
# Description  : MiniOS Debug Log interface
# Parameter    : 1) Debug  message
# Date         : <03-03-2014>
#==============================================================================
function g_LOG_Debug()
{
    LOG_WriteLog ${CON_DEBUG} ${LINENO} "$@"
    return $?
}

function g_LOG_Tip()
{
    while read LINE
    do
        logger -it osHealthCheck -p warning "TIP  :`basename $0` "[line_no=${LINENO}]:" $LINE"
    done
    return 0
}

####################################################
# function name:             g_LOG_Save_Result
# description:               generate the log with xml format
# parameter description:     $1 the check ID
#                            $2 the check item
#                            $3 the check result
#                            $4 the check message
# return value description:  None
####################################################
function g_LOG_Save_Result()
{
    Check_ID=$1
    Check_mod=$2
    Check_result=$3
    Check_info=$4

    # If the log dir is not exist, send the error message to log,
    # and return the error number.
    if [ ! -d ${RESULT_DIR} ]; then
        mkdir -p ${RESULT_DIR}
        chmod 700 ${RESULT_DIR}
        if [ $? -ne 0 ]; then
            echo  "[Error]Dir ${RESULT_DIR} create failed!"
            exit ${DIR_CREATE_FAILED}
        fi
    fi

    Log_name="${Check_ID}_itemresult.xml"
    echo "<item>" > ${RESULT_DIR}/${Log_name}
    chmod 600 ${RESULT_DIR}/${Log_name}
    # Check the space of "OS_LOG_PATH", if empty, send the error message to message,
    # and return the error number.
    if [ $? -eq 1 ]; then
        mountdiskused=`df ${OS_LOG_PATH} | grep -v Filesystem | awk '{print int($5)}'`
        if [ $mountdiskused -eq 100 ]; then
	    echo "[Warning]: No space left on ${OS_LOG_PATH} !"
            exit ${NO_SPACE_ERR}
        else
            echo "[Waring]:Can't write message to ${OS_LOG_PATH} !"
            exit ${DIR_CREATE_FAILED}
        fi
    fi
 
    echo "<id>${Check_ID}</id>" >> ${RESULT_DIR}/${Log_name}
    echo "<name>${Check_mod}</name>" >> ${RESULT_DIR}/${Log_name}
    echo "<result>${Check_result}</result>" >> ${RESULT_DIR}/${Log_name}
    echo "<output>${Check_info}</output>" >> ${RESULT_DIR}/${Log_name}
    echo "</item>" >> ${RESULT_DIR}/${Log_name}
    # Check the space of "OS_LOG_PATH" again.
    if [ $? -eq 1 ]; then
        mountdiskused=`df ${OS_LOG_PATH} | grep -v Filesystem | awk '{print int($5)}'`
        if [ $mountdiskused -eq 100 ]; then
            echo "[Warning]: No space left on ${OS_LOG_PATH} !"
            exit ${NO_SPACE_ERR}
        else
            echo "[Waring]:Can't write message to ${OS_LOG_PATH} !"
            exit ${DIR_CREATE_FAILED}
        fi
    fi
    sync ${RESULT_DIR}/${Log_name}
    if [ $? -ne 0 ];then
        echo "sync ${RESULT_DIR}/${Log_name} failed"
        g_LOG_Error "sync ${RESULT_DIR}/${Log_name} failed"
    fi
}

####################################################
# function name:             g_LOG_Save_Report
# description:               generate the report with xml format
# parameter description:     $1 the total check IDs
#                            $2 the failed check IDs
#                            $3 the passed check IDs
# return value description:  None
####################################################
function g_LOG_Save_Report()
{
    TOTAL_IDS=$1
    FAILED_IDS=$2
    PASSED_IDS=$3

    # If the log dir is not exist, send the error message to log,
    # and return the error number.
    if [ ! -d ${OS_LOG_PATH} ]; then
        mkdir -p ${OS_LOG_PATH}
        chmod 700 ${OS_LOG_PATH}
        if [ $? -ne 0 ]; then
            echo  "[Error]Dir ${OS_LOG_PATH} create failed!"
            exit ${DIR_CREATE_FAILED}
        fi
    fi

    echo '<?xml version="1.0" encoding="UTF-8"?>' > ${REPORT_FILE}
    echo "<Report>" >> ${REPORT_FILE}
    chmod 600 ${REPORT_FILE}
    # Check the space of "OS_LOG_PATH", if empty, send the error message to message,
    # and return the error number.
    if [ $? -eq 1 ]; then
        mountdiskused=`df ${OS_LOG_PATH} | grep -v Filesystem | awk '{print int($5)}'`
        if [ $mountdiskused -eq 100 ]; then
            echo "[Warning]: No space left on ${OS_LOG_PATH} !"
            exit ${NO_SPACE_ERR}
        else
            echo "[Waring]:Can't write message to ${OS_LOG_PATH} !"
            exit ${DIR_CREATE_FAILED}
        fi
    fi
    TOTAL_IDS=$(echo ${TOTAL_IDS} | sed 's/[ ]\{1,\}/,/g')
    FAILED_IDS=$(echo ${FAILED_IDS} | sed 's/[ ]\{1,\}/,/g')
    PASSED_IDS=$(echo ${PASSED_IDS} | sed 's/[ ]\{1,\}/,/g')

    echo "  <TotalItems>${TOTAL_IDS}</TotalItems>" >> ${REPORT_FILE}
    echo "  <FailedItems>${FAILED_IDS}</FailedItems>" >> ${REPORT_FILE}
    echo "  <PassedItems>${PASSED_IDS}</PassedItems>" >> ${REPORT_FILE}
    echo "</Report>" >> ${REPORT_FILE}
    # Check the space of "OS_LOG_PATH" again.
    if [ $? -eq 1 ]; then
        mountdiskused=`df ${OS_LOG_PATH} | grep -v Filesystem | awk '{print int($5)}'`
        if [ $mountdiskused -eq 100 ]; then
            echo "[Warning]: No space left on ${OS_LOG_PATH} !"
            exit ${NO_SPACE_ERR}
        else
            echo "[Waring]:Can't write message to ${OS_LOG_PATH} !"
            exit ${DIR_CREATE_FAILED}
        fi
    fi
    sync ${REPORT_FILE}
    if [ $? -ne 0 ];then
        echo "sync ${REPORT_FILE} failed"
        g_LOG_Error "sync ${REPORT_FILE} failed"
    fi
}

g_LOG_Init "$@"
